@@ -3,29 +3,21 @@
 package KiokuDB::Backend::DBI;
 use Moose;
 
-use Moose::Util::TypeConstraints;
+use MooseX::Types -declare => [qw(ValidColumnName)];
 
-use MooseX::Types -declare => [qw(ValidColumnName SchemaProto)];
-
-use MooseX::Types::Moose qw(ArrayRef HashRef Str Defined);
+use MooseX::Types::Moose qw(ArrayRef HashRef Str);
 
 use Moose::Util::TypeConstraints qw(enum);
 
 use Data::Stream::Bulk::DBI;
-use SQL::Abstract;
-use JSON;
-use Scalar::Util qw(weaken);
 
 use KiokuDB::Backend::DBI::Schema;
-use KiokuDB::TypeMap;
-use KiokuDB::TypeMap::Entry::DBIC::Row;
-use KiokuDB::TypeMap::Entry::DBIC::ResultSourceHandle;
-use KiokuDB::TypeMap::Entry::DBIC::ResultSet;
-use KiokuDB::TypeMap::Entry::DBIC::Schema;
+
+use SQL::Abstract;
 
 use namespace::clean -except => 'meta';
 
-our $VERSION = "1.11_02";
+our $VERSION = "1.10";
 
 with qw(
     KiokuDB::Backend
@@ -46,7 +38,6 @@ my @reserved_cols = ( @std_cols, 'data' );
 my %reserved_cols = ( map { $_ => 1 } @reserved_cols );
 
 subtype ValidColumnName, as Str, where { not exists $reserved_cols{$_} };
-subtype SchemaProto, as Defined, where { !ref($_) || blessed($_) and $_->isa("DBIx::Class::Schema::KiokuDB") };
 
 sub new_from_dsn {
     my ( $self, $dsn, @args ) = @_;
@@ -65,12 +56,6 @@ sub BUILD {
 
 has '+serializer' => ( default => "json" ); # to make dumps readable
 
-has json => (
-    isa => "Object",
-    is  => "ro",
-    default => sub { JSON->new },
-);
-
 has create => (
     isa => "Bool",
     is  => "ro",
@@ -82,6 +67,7 @@ has 'dsn' => (
     is  => "ro",
 );
 
+
 has [qw(user password)] => (
     isa => "Str",
     is  => "ro",
@@ -168,13 +154,7 @@ has schema => (
     isa => "DBIx::Class::Schema",
     is  => "ro",
     lazy_build => 1,
-    handles => [qw(deploy kiokudb_entries_source_name)],
-);
-
-has schema_proto => (
-    isa => SchemaProto,
-    is  => "ro",
-    default => "KiokuDB::Backend::DBI::Schema",
+    handles => [qw(deploy)],
 );
 
 has schema_hook => (
@@ -186,11 +166,9 @@ has schema_hook => (
 sub _build_schema {
     my $self = shift;
 
-    my $schema = $self->schema_proto->clone;
+    my $schema = KiokuDB::Backend::DBI::Schema->clone;
 
-    unless ( $schema->kiokudb_entries_source_name ) {
-        $schema->define_kiokudb_schema( extra_entries_columns => $self->columns );
-    }
+    $schema->source("entries")->add_columns(@{ $self->columns });
 
     if ( $self->has_schema_hook ) {
         my $h = $self->schema_hook;
@@ -209,28 +187,6 @@ has storage => (
 
 sub _build_storage { shift->schema->storage }
 
-has for_update => (
-    isa => "Bool",
-    is  => "ro",
-    default => 1,
-);
-
-has _for_update => (
-    isa => "Bool",
-    is  => "ro",
-    lazy_build => 1,
-);
-
-sub _build__for_update {
-    my $self = shift;
-
-    return (
-        $self->for_update
-            and
-        $self->storage->sqlt_type =~ /^(?:MySQL|Oracle|PostgreSQL)$/
-    );
-}
-
 has columns => (
     isa => ArrayRef[ValidColumnName|HashRef],
     is  => "ro",
@@ -246,7 +202,7 @@ has _columns => (
 sub _build__columns {
     my $self = shift;
 
-    my $rs = $self->schema->source( $self->kiokudb_entries_source_name );
+    my $rs = $self->schema->source("entries");
 
     my @user_cols = grep { not exists $reserved_cols{$_} } $rs->columns;
 
@@ -293,28 +249,6 @@ sub _build_sql_abstract {
     SQL::Abstract->new;
 }
 
-sub register_handle {
-    my ( $self, $kiokudb ) = @_;
-
-    $self->schema->_kiokudb_handle($kiokudb);
-}
-
-sub default_typemap {
-    KiokuDB::TypeMap->new(
-        isa_entries => {
-            # redirect to schema row
-            'DBIx::Class::Row'                => KiokuDB::TypeMap::Entry::DBIC::Row->new,
-
-            # actual serialization
-            'DBIx::Class::ResultSet'          => KiokuDB::TypeMap::Entry::DBIC::ResultSet->new,
-
-            # fake, the entries never get written to the db
-            'DBIx::Class::ResultSourceHandle' => KiokuDB::TypeMap::Entry::DBIC::ResultSourceHandle->new,
-            'DBIx::Class::Schema'             => KiokuDB::TypeMap::Entry::DBIC::Schema->new,
-        },
-    );
-}
-
 sub insert {
     my ( $self, @entries ) = @_;
 
@@ -342,7 +276,7 @@ sub insert {
 sub entries_to_rows {
     my ( $self, @entries ) = @_;
 
-    my ( %insert, %update, @dbic );
+    my ( %insert, %update );
 
     foreach my $t ( \%insert, \%update ) {
         foreach my $col ( @{ $self->_ordered_columns } ) {
@@ -351,20 +285,12 @@ sub entries_to_rows {
     }
 
     foreach my $entry ( @entries ) {
-        my $id = $entry->id;
+        my $targ = $entry->prev ? \%update : \%insert;
 
-        if ( $id =~ /^dbic:schema:/ ) {
-            next;
-        } elsif ( $id =~ /^dbic:row:/ ) {
-            push @dbic, $entry->data;
-        } else {
-            my $targ = $entry->prev ? \%update : \%insert;
-
-            my $row = $self->entry_to_row($entry, $targ);
-        }
+        my $row = $self->entry_to_row($entry, $targ);
     }
 
-    return \( %insert, %update, @dbic );
+    return \( %insert, %update );
 }
 
 sub entry_to_row {
@@ -399,7 +325,7 @@ sub entry_to_row {
 }
 
 sub insert_rows {
-    my ( $self, $insert, $update, $dbic ) = @_;
+    my ( $self, $insert, $update ) = @_;
 
     $self->dbh_do(sub {
         my ( $storage, $dbh ) = @_;
@@ -441,17 +367,9 @@ sub insert_rows {
 
             $sth->finish;
         }
-
-        $_->insert_or_update for @$dbic;
     });
 }
 
-sub prepare_select {
-    my ( $self, $dbh, $stmt ) = @_;
-
-    $dbh->prepare_cached($stmt . ( $self->_for_update ? " FOR UPDATE" : "" ));
-}
-
 sub prepare_insert {
     my ( $self, $dbh ) = @_;
 
@@ -493,123 +411,39 @@ sub update_index {
 }
 
 sub get {
-    my ( $self, @rows_and_ids ) = @_;
+    my ( $self, @ids ) = @_;
 
     my %entries;
 
-    my ( @rows, @ids );
+    $self->dbh_do(sub {
+        my ( $storage, $dbh ) = @_;
 
-    for ( @rows_and_ids ) {
-        if ( /^dbic:schema/ ) {
-            # schema objects aren't actually written into the DB
-            # FIXME special case FSCK
-            $entries{$_} = KiokuDB::Entry->new(
-                id    => $_,
-                ( $_ eq 'dbic:schema'
-                    ? ( class => 'DBIx::Class::Schema', data => $self->schema )
-                    : ( class => 'DBIx::Class::ResultSourceHandle', data => undef )
-                )
-            );
-        } elsif ( /^dbic:row:/ ) {
-            push @rows, $_;
-        } else {
-            push @ids, $_;
-        }
-    }
+        my $sth;
 
-    if ( @ids ) {
-        $self->dbh_do(sub {
-            my ( $storage, $dbh ) = @_;
-
-            my $sth;
-
-            $sth = $self->prepare_select($dbh, "SELECT id, data FROM entries WHERE id IN (" . join(", ", ('?') x @ids) . ")");
+        if ( @ids ) {
+            $sth = $dbh->prepare_cached("SELECT id, data FROM entries WHERE id IN (" . join(", ", ('?') x @ids) . ")");
             $sth->execute(@ids);
-
-            $sth->bind_columns( \my ( $id, $data ) );
-
-            # not actually necessary but i'm keeping it around for reference:
-            #my ( $id, $data );
-            #use DBD::Pg qw(PG_BYTEA);
-            #$sth->bind_col(1, \$id);
-            #$sth->bind_col(2, \$data, { pg_type => PG_BYTEA });
-
-            while ( $sth->fetch ) {
-                $entries{$id} = $data;
-            }
-        });
-    }
-
-    if ( @rows ) {
-        my $schema = $self->schema;
-        my $json = $self->json;
-
-        my ( %keys, %ids );
-
-        foreach my $id ( @rows ) {
-            my ( $rs_name, @key ) = @{ $json->decode(substr($id,length('dbic:row:'))) };
-
-            if ( @key > 2 ) {
-                # multi column primary keys need 'find'
-                my $obj = $schema->resultset($rs_name)->find(@key);
-
-                $entries{$id} = KiokuDB::Entry->new(
-                    id    => $id,
-                    class => ref($obj),
-                    data  => $obj,
-                );
-            } else {
-                # for other objects we queue up IDs for a single SELECT
-                push @{ $keys{$rs_name} ||= [] }, $key[0];
-                push @{ $ids{$rs_name}  ||= [] }, $id;
-            }
+        } else {
+            $sth = $dbh->prepare_cached("SELECT id, data FROM entries");
+            $sth->execute;
         }
 
-        foreach my $rs_name ( keys %keys ) {
-            my $rs = $schema->resultset($rs_name);
+        $sth->bind_columns( \my ( $id, $data ) );
 
-            my $ids = $ids{$rs_name};
+        # not actually necessary but i'm keeping it around for reference:
+        #my ( $id, $data );
+        #use DBD::Pg qw(PG_BYTEA);
+        #$sth->bind_col(1, \$id);
+        #$sth->bind_col(2, \$data, { pg_type => PG_BYTEA });
 
-            my @objs;
-
-            if ( @$ids == 1 ) {
-                my $id = $ids->[0];
-
-                my $obj = $rs->find($keys{$rs_name}[0]) or return;
-
-                $entries{$id} = KiokuDB::Entry->new(
-                    id => $id,
-                    class => ref($obj),
-                    data => $obj,
-                );
-            } else {
-                my @pk = $rs->result_source->primary_columns;
-
-                my $keys = $keys{$rs_name};
-
-                my @objs = $rs->search({ $pk[0] => $keys })->all;
-
-                return if @objs != @$ids;
-
-                # this key lookup is because it's not returned in the same order
-                my %pk_to_id;
-                @pk_to_id{@{ $keys{$rs_name} }} = @$ids;
-
-                foreach my $obj ( @objs ) {
-                    my $id = $pk_to_id{$obj->id};
-                    $entries{$id} = KiokuDB::Entry->new(
-                        id    => $id,
-                        class => ref($obj),
-                        data  => $obj,
-                    );
-                }
-            }
+        while ( $sth->fetch ) {
+            $entries{$id} = $data;
         }
-    }
+    });
 
-    return if @rows_and_ids != keys %entries; # ->rows only works after we're done
+    return if @ids != keys %entries; # ->rows only works after we're done
 
-    map { ref($_) ? $_ : $self->deserialize($_) } @entries{@rows_and_ids};
+    return map { $self->deserialize($_) } @entries{@ids};
 }
 
 sub delete {
@@ -643,7 +477,7 @@ sub exists {
     $self->dbh_do(sub {
         my ( $storage, $dbh ) = @_;
 
-        my $sth = $self->prepare_select($dbh, "SELECT id FROM entries WHERE id IN (" . join(", ", ('?') x @ids) . ")");
+        my $sth = $dbh->prepare_cached("SELECT id FROM entries WHERE id IN (" . join(", ", ('?') x @ids) . ")");
         $sth->execute(@ids);
 
         $sth->bind_columns( \( my $id ) );
@@ -884,16 +718,6 @@ will be C<NULL>.
 These columns are only used for lookup purposes, only C<data> is consulted when
 loading entries.
 
-=head1 DBIC INTEGRATION
-
-This backend is layered on top of L<DBIx::Class::Storage::DBI> and reused
-L<DBIx::Class::Schema> for DDL.
-
-Because of this objects from a L<DBIx::Class::Schema> can refer to objects in
-the KiokuDB entries table, and vice versa.
-
-For more details see L<DBIx::Class::Schema::KiokuDB>.
-
 =head1 SUPPORTED DATABASES
 
 This driver has been tested with MySQL 5 (4.1 should be the minimal supported
@@ -1015,16 +839,6 @@ argument just before connecting.
 If you need to modify the schema in some way (adding indexes or constraints)
 this is where it should be done.
 
-=item for_update
-
-If true (the defaults), will cause all select statement to be issued with a
-C<FOR UPDATE> modifier on MySQL, Postgres and Oracle.
-
-This is highly reccomended because these database provide low isolation
-guarantees as configured out the box, and highly interlinked graph databases
-are much more susceptible to corruption because of lack of transcational
-isolation than normalized relational databases.
-
 =item sqlite_sync_mode
 
 If this attribute is set and the underlying database is SQLite, then
@@ -1084,23 +898,6 @@ Drops the C<entries> and C<gin_index> tables.
 
 =back
 
-=head1 TROUBLESHOOTING
-
-=head2 I get C<unexpected end of string while parsing JSON string>
-
-You are problably using MySQL, which comes with a helpful data compression
-feature: when your serialized objects are larger than the maximum size of a
-C<BLOB> column MySQL will simply shorten it for you.
-
-Why C<BLOB> defaults to 64k, and how on earth someone would consider silent
-data truncation a sane default I could never fathom, but nevertheless MySQL
-does allow you to disable this by setting the "strict" SQL mode in the
-configuration.
-
-To resolve the actual problem (though this obviously won't repair your lost
-data), alter the entries table so that the C<data> column uses the nonstandard
-C<LONGBLOB> datatype.
-
 =head1 VERSION CONTROL
 
 L<http://github.com/nothingmuch/kiokudb-backend-dbi>
